function createDefaultContrastSet(cfg)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function creates a default-set of t-contrasts that can be used for
% grid code analysis and to derive grid code metrics

% Load data from SPM.mat
SPMmatFile = [cfg.GLM.dataDir filesep 'SPM.mat'];
SPMmatFileInfo = load(SPMmatFile);

matlabbatch{1}.spm.stats.con.consess = []; % default init

% get info about which regressor is estimable (uniquely defined, as shown in the design matrix)
parameterEstimability = spm_SpUtil('IsCon', SPMmatFileInfo.SPM.xX.X);

% contrasts are created for each grid event regressor individually
gridEventNames = getGridEventNames(cfg.eventInfo);
for gridEventTypeIdx = 1:length(gridEventNames)
    
    % contrasts are created for each mask individually
    for maskIdx = 1:length(cfg.GLM.GLM2_roiMask_calcMeanGridOri)

        % default init
        regNr_pmod = [];
        regNr_aligned = [];
        regNr_misaligned = [];
        peak = [];
        
        % contrasts are created for each run individually
        for runIdx = 1:length(cfg.eventInfo.run)
           
            % do we expect to find a regressor for the current event type in this run?
            if any(strcmp({cfg.eventInfo.run(runIdx).gridEventType(:).eventName}, gridEventNames{gridEventTypeIdx}))
            
                contrastVector = zeros(1,length(SPMmatFileInfo.SPM.Vbeta)); % default init (all regressors 0)

                if strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'pmod')

                    regNrFound_pmod = find(~cellfun(@isempty,regexp({SPMmatFileInfo.SPM.Vbeta(:).descrip}, regexptranslate('wildcard', ['Sn(' num2str(runIdx) ') GridEvent-' gridEventNames{gridEventTypeIdx} 'x-PMOD-alignmentWithROI' num2str(maskIdx) '*bf(1)']))));

                    % does the pmod regressor exist and is it estimable?
                    if ~isempty(regNrFound_pmod) && parameterEstimability(regNrFound_pmod)

                        % define new contrast for spm matlabbatch. contrastVector: regressor for pmod...1, other regressors...0 
                        regNr_pmod(runIdx) = regNrFound_pmod;
                        contrastVector(regNr_pmod(runIdx)) = 1;
                        matlabbatch = addSPMtconToMatlabbatch(matlabbatch, contrastVector, ['Run' num2str(runIdx) '_' gridEventNames{gridEventTypeIdx} '-PMOD-alignmentWithROI' num2str(maskIdx) 'meanOri']);
                    else
                        disp(['Contrast could not be generated for run' num2str(runIdx) ' ''' gridEventNames{gridEventTypeIdx} ''' PMOD-alignment with ROI' num2str(maskIdx) ' mean grid orientation. Regressor does not exist or is not estimable.']); 
                    end

                elseif strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'aligned_misaligned')

                    regNrFound_aligned    = find(~cellfun(@isempty,regexp({SPMmatFileInfo.SPM.Vbeta(:).descrip}, regexptranslate('wildcard', ['Sn(' num2str(runIdx) ') GridEvent-' gridEventNames{gridEventTypeIdx} '-alignedWithROI' num2str(maskIdx) '*bf(1)']))));
                    regNrFound_misaligned = find(~cellfun(@isempty,regexp({SPMmatFileInfo.SPM.Vbeta(:).descrip}, regexptranslate('wildcard', ['Sn(' num2str(runIdx) ') GridEvent-' gridEventNames{gridEventTypeIdx} '-misalignedWithROI' num2str(maskIdx) '*bf(1)']))));

                    % do both regressors exist and are they estimable?
                    if ~isempty(regNrFound_aligned) && ~isempty(regNrFound_misaligned) && all(parameterEstimability([regNrFound_misaligned, regNrFound_aligned]))

                        % the regressors both exist, so we define a new contrast for spm matlabbatch. contrastVector: regressor for aligned runs...1, regressor for misaligned runs...-1, other regressors...0
                        regNr_aligned(runIdx) = regNrFound_aligned;
                        regNr_misaligned(runIdx) = regNrFound_misaligned;
                        contrastVector(regNrFound_aligned) = 1;
                        contrastVector(regNrFound_misaligned) = -1;
                        matlabbatch = addSPMtconToMatlabbatch(matlabbatch, contrastVector, ['Run' num2str(runIdx) '_' gridEventNames{gridEventTypeIdx} '-alignedVSmisalignedWithROI' num2str(maskIdx) 'meanOri']);
                    else
                        disp(['Contrast could not be generated for run' num2str(runIdx) ' ''' gridEventNames{gridEventTypeIdx} ''' aligned/misaligned with ROI' num2str(maskIdx) ' mean grid orientation. One or both regressors do not exist or are not estimable.']); 
                    end

                elseif strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'aligned_misaligned_multiple')

                    % at which orientation values (currentOri-meanOri) do we expect peaks in activation (both positive and negative peaks)?
                    expectedPeaks = 0 : 2*pi/(cfg.GLM.xFoldSymmetry*2) : 2*pi;
                    expectedPeaks(end) = []; % first peak (0 degrees) = last peak (360 degrees)

                    for peakIdx = 1:length(expectedPeaks)

                        % what is the current peak-bin?
                        peakStr_deg = num2str(round(expectedPeaks(peakIdx)*180/pi));

                        % default initialization of contras vector
                        contrastVector = zeros(1,length(SPMmatFileInfo.SPM.Vbeta)); % default init (all regressors 0)

                        % find the regressor number for the current peak-bin
                        regNrFound = find(~cellfun(@isempty,regexp({SPMmatFileInfo.SPM.Vbeta(:).descrip}, regexptranslate('wildcard', ['Sn(' num2str(runIdx) ') GridEvent-' gridEventNames{gridEventTypeIdx} '-' peakStr_deg 'degPeak-*WithROI' num2str(maskIdx) '*bf(1)']))));

                        % does the regressor exist and is it estimable?
                        if ~isempty(regNrFound) && parameterEstimability(regNrFound)

                            % the regressor does exist, so we define a new contrast for spm matlabbatch. contrastVector: regressor for current peak-bin = 1, other regressors...0
                            peak(peakIdx).regNr(runIdx) = regNrFound;
                            contrastVector(regNrFound) = 1;
                            if mod(peakIdx,2); alignmentStr = 'aligned'; else; alignmentStr = 'misaligned'; end
                            matlabbatch = addSPMtconToMatlabbatch(matlabbatch, contrastVector, ['Run' num2str(runIdx) '_' gridEventNames{gridEventTypeIdx} '-' peakStr_deg 'degPeak-' alignmentStr 'WithROI' num2str(maskIdx) 'meanOri']);
                        else
                            disp(['Contrast could not be generated for run' num2str(runIdx) ' ''' gridEventNames{gridEventTypeIdx} ''' ' peakStr_deg ' degrees peak (aligned/misaligned with ROI' num2str(maskIdx) ' mean grid ori). Regressor does not exist or is not estimable.']); 
                        end
                    end
                end
            end
        end

        % now we have created contrasts for this grid event type, this mask, for all runs individually
        % so we can now create a contrast with all the regressors of this event type/mask, across all runs
        if strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'pmod')

            % add contrast to matlabbatch
            contrastVector = zeros(1,length(SPMmatFileInfo.SPM.Vbeta)); % default init (all regressors 0)
            if length(regNr_pmod(regNr_pmod>0))>1
                contrastVector(regNr_pmod(regNr_pmod>0)) = 1;
                matlabbatch = addSPMtconToMatlabbatch(matlabbatch, contrastVector, ['allRuns_' gridEventNames{gridEventTypeIdx} '-PMOD-alignmentWithROI' num2str(maskIdx) 'meanOri']);
            end
            
        elseif strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'aligned_misaligned')

            % add contrast to matlabbatch
            contrastVector = zeros(1,length(SPMmatFileInfo.SPM.Vbeta)); % default init (all regressors 0)
            if length(regNr_aligned(regNr_aligned>0))>1 && length(regNr_misaligned(regNr_misaligned>0))>1
                contrastVector(regNr_aligned(regNr_aligned>0)) = 1;
                contrastVector(regNr_misaligned(regNr_misaligned>0)) = -1;
                matlabbatch = addSPMtconToMatlabbatch(matlabbatch, contrastVector, ['allRuns_' gridEventNames{gridEventTypeIdx} '-alignedVSmisalignedWithROI' num2str(maskIdx) 'meanOri']);
            end

        elseif strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'aligned_misaligned_multiple')

            for peakIdx = 1:length(expectedPeaks)

                % what is the current peak-bin?
                peakStr_deg = num2str(round(expectedPeaks(peakIdx)*180/pi));
                if mod(peakIdx,2); alignmentStr = 'aligned'; else; alignmentStr = 'misaligned'; end

                % add contrast to matlabbatch
                contrastVector = zeros(1,length(SPMmatFileInfo.SPM.Vbeta)); % default init (all regressors 0)
                if ~isempty(peak) && ~isempty(peak(peakIdx).regNr) && length(peak(peakIdx).regNr(peak(peakIdx).regNr>0))>1
                    contrastVector(peak(peakIdx).regNr(peak(peakIdx).regNr>0)) = 1;
                    matlabbatch = addSPMtconToMatlabbatch(matlabbatch, contrastVector, ['allRuns_' gridEventNames{gridEventTypeIdx} '-' peakStr_deg 'degPeak-' alignmentStr 'WithROI' num2str(maskIdx) 'meanOri']);
                end
            end
        end
    end
end


% define spm matlabbatch
matlabbatch{1}.spm.stats.con.spmmat = {SPMmatFile};
matlabbatch{1}.spm.stats.con.delete = 0;

% run spm matlabbatch
spm_jobman('initcfg');
spm_jobman('run', matlabbatch);



