function cfg = designParameters2cfg(handles, GLMnr)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function reads settings from the GridCAT GUI and transforms it into
% the configuration variable cfg that is needed for GLM1 and/or GLM2.
% An empty variable cfg will be returned, if parameters are not specified correctly.


%% SAVE GLM NUMBER

cfg.GLMnr = GLMnr;


%% FUNCTIONAL SCANS, EVENT-TABLES, ADDITIONAL REGRESSORS

try
    tableData = getUserData(handles.uitable_filelist, 'UserData');
    for runIdx = 1:length(tableData.run)
        cfg.rawData.run(runIdx).functionalScans = tableData.run(runIdx).functionalScans;        
        cfg.rawData.run(runIdx).eventTable_file = tableData.run(runIdx).eventTable;
        if isempty(cfg.rawData.run(runIdx).functionalScans) || isempty(cfg.rawData.run(runIdx).eventTable_file)
            error(' ');
        end
        if isfield(tableData.run(runIdx)', 'additionalRegressorsFile') && ~isempty(tableData.run(runIdx).additionalRegressorsFile)
            cfg.rawData.run(runIdx).additionalRegressors_file = tableData.run(runIdx).additionalRegressorsFile;
        end
    end
catch
    h = msgbox('Functional scans and/or event-tables are not specified correctly.');
    adaptFigurePosition(h);
    cfg = [];
    return
end
    

%% SET PATHS AND DIRECTORIES

if GLMnr == 1

    cfg.GLM.dataDir = cell2mat(get(handles.text_dataDirPath_GLM1, 'UserData'));
    if isempty(cfg.GLM.dataDir)
        h = msgbox(['The GLM' num2str(GLMnr) ' data directory is not specified correctly.']);
        adaptFigurePosition(h);
        cfg = [];
        return
    end

elseif GLMnr == 2

    cfg.GLM.dataDir = cell2mat(get(handles.text_dataDirPath_GLM2, 'UserData'));
    if isempty(cfg.GLM.dataDir)
        h = msgbox(['The GLM' num2str(GLMnr) ' data directory is not specified correctly.']);
        adaptFigurePosition(h);
        cfg = [];
        return
    end

    cfg.GLM.GLM1_resultsDir = cell2mat(get(handles.text_dataDirPath_GLM1, 'UserData'));
    if isempty(cfg.GLM.GLM1_resultsDir)
        h = msgbox('A valid GLM1 data directory needs to be specified also when computing GLM2, since voxel-wise orientation data from the GLM1 data directory is used during GLM2 calculations.');
        adaptFigurePosition(h);
        cfg = [];
        return
    end
end


%% MODEL SETTINGS

% TR
cfg.GLM.TR = str2double(get(handles.edit_TR, 'String'));
if isnan(cfg.GLM.TR)
    h = msgbox('The interscan-interval is not specified correctly.');
    adaptFigurePosition(h);
    cfg = [];
    return
end
    
% x-fold symmetry value that the model is testing
cfg.GLM.xFoldSymmetry = str2double(get(handles.edit_xFoldSymmetry, 'String'));
if isnan(cfg.GLM.xFoldSymmetry) || ~isreal(cfg.GLM.xFoldSymmetry)
    h = msgbox('The grid cody symmetry value is not specified correctly.');
    adaptFigurePosition(h);
    cfg = [];
    return
end


% optional: do you want to display the design matrix after it has been created?
% if this is not defined, then the design matrix will not be displayed
cfg.GLM.dispDesignMatrix = 1;


% -------------------------------
% ADVANCED MODEL SETTINGS
% -------------------------------

% read values for advanced model settings from user data of main figure
ud_mainFigure = getUserData(handles.figure_GridCAT_GUI, 'UserData');

% Masking threshold
cfg.GLM.maskingThreshold = ud_mainFigure.advancedModelSettings.maskingThreshold;
if isnan(cfg.GLM.maskingThreshold) || ~isreal(cfg.GLM.maskingThreshold) || cfg.GLM.maskingThreshold<0 || cfg.GLM.maskingThreshold>1
    h = msgbox('The masking threshold is not specified correctly. Please check advanced scanning parameters and model settings.');
    adaptFigurePosition(h);
    cfg = [];
    return
end

% Microtime onset
cfg.GLM.microtimeOnset = ud_mainFigure.advancedModelSettings.microtimeOnset;
if isnan(cfg.GLM.microtimeOnset) || ~isreal(cfg.GLM.microtimeOnset)
    h = msgbox('The microtime onset is not specified correctly. Please check advanced scanning parameters and model settings.');
    adaptFigurePosition(h);
    cfg = [];
    return
end

% Microtime resolution
cfg.GLM.microtimeResolution = ud_mainFigure.advancedModelSettings.microtimeResolution;
if isnan(cfg.GLM.microtimeResolution) || ~isreal(cfg.GLM.microtimeResolution)
    h = msgbox('The microtime resolution is not specified correctly. Please check advanced scanning parameters and model settings.');
    adaptFigurePosition(h);
    cfg = [];
    return
end

% HPF per run
try
    cfg.GLM.HPF_perRun = eval(['[' ud_mainFigure.advancedModelSettings.HPF_str ']']);
    if length(cfg.GLM.HPF_perRun) == 1
        cfg.GLM.HPF_perRun = repmat(cfg.GLM.HPF_perRun, 1, length(tableData.run));
    end
    if length(cfg.GLM.HPF_perRun) ~= length(cfg.rawData.run)
        error(' ');
    end
catch
    h = msgbox('The high-pass filter is not specified correctly. Please check advanced scanning parameters and model settings.');
    adaptFigurePosition(h);
    cfg = [];
    return
end

% Model derivatives
cfg.GLM.derivatives = ud_mainFigure.advancedModelSettings.derivatives; 


%% SPECIFY GRID EVENT USAGE

% keep grid events that are not used for this analysis?
% which grid events should be modeled for this GLM?
if GLMnr == 1
    cfg.GLM.keepUnusedGridEvents = get(handles.checkbox_keepUnusedEvents_GLM1, 'Value');
    cfg.GLM.eventUsageSpecifier = get(handles.popupmenu_dataSelection_GLM1, 'Value');
elseif GLMnr == 2
    cfg.GLM.keepUnusedGridEvents = get(handles.checkbox_keepUnusedEvents_GLM2, 'Value');
    cfg.GLM.eventUsageSpecifier = get(handles.popupmenu_dataSelection_GLM2, 'Value');
end
if cfg.GLM.eventUsageSpecifier == 1
    h = msgbox('Before computing a GLM, please select grid events to be used.');
    adaptFigurePosition(h);
    cfg = [];
    return
end


%% SPECIFIC SETTINGS FOR GLM2

if GLMnr == 2

    % Voxels within this mask are used to calculate the mean grid orientation
    cfg.GLM.GLM2_roiMask_calcMeanGridOri = getUserData(handles.text_maskImg_meanGridOri, 'UserData'); % the mask must be aligned with the (mean)EPI images
    if isempty(cfg.GLM.GLM2_roiMask_calcMeanGridOri)
        h = msgbox('The ROI mask for calculating the mean grid orientation is not specified correctly.');
        adaptFigurePosition(h);
        cfg = [];
        return
    end

    % use different weighting for individual voxels within the ROI?
    cfg.GLM.GLM2_useWeightingForVoxels = get(handles.checkbox_useWeightingForVoxels, 'Value');

    % calculate the mean grid orientation within an ROI across all runs or separately for each run?
    cfg.GLM.GLM2_averageMeanGridOriAcrossRuns = get(handles.checkbox_averageMeanGridOriAcrossRuns, 'Value');

    % Which kind of grid event regressor should be added to the model?
    switch get(handles.popupmenu_gridRegressors, 'Value')
        case 1
            h = msgbox('Before computing GLM2, please select which type of grid code regressor you want to include in the model.');
            adaptFigurePosition(h);
            cfg = [];
            return
        case 2
            cfg.GLM.GLM2_gridRegressorMethod = 'pmod';
        case 3
            cfg.GLM.GLM2_gridRegressorMethod = 'aligned_misaligned'; 
        case 4
            cfg.GLM.GLM2_gridRegressorMethod = 'aligned_misaligned_multiple';
    end
end


%% Get GUI parameters (so that it can be saved on mat-file and later restored)

cfg.GUIdata = getDesignParameters(handles);

