function gridEventType = estimateGridOri_voxelwise(cfg)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function estimates the voxel-wise grid orientations based on the
% parameter estimates of a grid event's PMOD regressors (sin & cos)

% Load data from SPM.mat
SPMmatFileInfo = load([cfg.GLM.dataDir filesep 'SPM.mat']);

% calculations are done for each grid event regressor individually
gridEventNames = getGridEventNames(cfg.eventInfo);
for gridEventTypeIdx = 1:length(gridEventNames)
    
    % Save name of current grid event type
    gridEventType(gridEventTypeIdx).eventName = gridEventNames{gridEventTypeIdx};

    % calculations are done for each run individually
    for runIdx = 1:length(cfg.eventInfo.run)

        % find beta images for the current grid event regressor's SIN/COS pmod
        pmodSuffixList_sin1cos2 = {'x-pmodSIN', 'x-pmodCOS'};
        for pmodIdx = 1:2

            regNr = find(~cellfun(@isempty,regexp({SPMmatFileInfo.SPM.Vbeta(:).descrip}, regexptranslate('wildcard', ['Sn(' num2str(runIdx) ') GridEvent-' gridEventNames{gridEventTypeIdx} pmodSuffixList_sin1cos2{pmodIdx} '*bf(1)']))));
            if ~isempty(regNr)
                [hdr, pmodBetaImages_sin1cos2(pmodIdx).vol(:,:,:,runIdx)] = loadImage_SPM([cfg.GLM.dataDir filesep SPMmatFileInfo.SPM.Vbeta(regNr).fname]);
                gridEventTypeExistsInRun(runIdx) = 1;
            else
                pmodBetaImages_sin1cos2(pmodIdx).vol(:,:,:,runIdx) = NaN(SPMmatFileInfo.SPM.xVol.DIM');
                gridEventTypeExistsInRun(runIdx) = 0;
            end

        end

        % use the beta images to estimate voxelwise grid orientations
        [gridEventType(gridEventTypeIdx).run(runIdx).oriPerVoxel_rad, gridEventType(gridEventTypeIdx).run(runIdx).amplitudePerVoxel] = betas2ori(cfg.GLM.xFoldSymmetry, pmodBetaImages_sin1cos2(1).vol(:,:,:,runIdx), pmodBetaImages_sin1cos2(2).vol(:,:,:,runIdx));

        % if this grid event type exists in this run, save voxelwise grid orientations (in degrees) and amplitudes as an image
        if ~isempty(regNr)
            % change voxelwise grid orientations from [rad] to [deg] and save it as an image
            saveImage_SPM(hdr, gridEventType(gridEventTypeIdx).run(runIdx).oriPerVoxel_rad*180/pi, [cfg.GLM.dataDir filesep 'voxelwiseGridOri_' gridEventNames{gridEventTypeIdx} '_run' num2str(runIdx) '_deg.nii']);
            saveImage_SPM(hdr, gridEventType(gridEventTypeIdx).run(runIdx).amplitudePerVoxel,      [cfg.GLM.dataDir filesep 'voxelwiseAmplitude_' gridEventNames{gridEventTypeIdx} '_run' num2str(runIdx) '.nii']);
        end

    end

    % now we have estimated voxelwise orientations for this grid event type, for all runs individually
    % so we can average the individual run's beta images here, in order to estimate voxelwise grid ori's across all runs for this regressor
    [gridEventType(gridEventTypeIdx).allRunsAvg.oriPerVoxel_rad, gridEventType(gridEventTypeIdx).allRunsAvg.amplitudePerVoxel] = betas2ori(cfg.GLM.xFoldSymmetry, nanmean(pmodBetaImages_sin1cos2(1).vol,4), nanmean(pmodBetaImages_sin1cos2(2).vol,4) );

    % if this grid event type is existent in two or more runs, save voxelwise grid orientations (in degrees) and amplitudes as an image
    if sum(gridEventTypeExistsInRun)>1
        saveImage_SPM(hdr, gridEventType(gridEventTypeIdx).allRunsAvg.oriPerVoxel_rad*180/pi, [cfg.GLM.dataDir filesep 'voxelwiseGridOri_' gridEventNames{gridEventTypeIdx} '_allRunsAvg_deg.nii']);
        saveImage_SPM(hdr, gridEventType(gridEventTypeIdx).allRunsAvg.amplitudePerVoxel,      [cfg.GLM.dataDir filesep 'voxelwiseAmplitude_' gridEventNames{gridEventTypeIdx} '_allRunsAvg.nii']);
    end
end


