function cfg = generateMultiCondFile(cfg)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function uses all informations about events, timing etc., in order to
% create 'multiple conditions files' for each run, which specify the GLM regressors.

% Create a separate multiple conditions file for each run
for runIdx = 1:length(cfg.eventInfo.run)

    % get variables (names, onsets, durations) as they are needed for the multiple conditions file in SPM
    names = cfg.eventInfo.run(runIdx).names;
    onsets = cfg.eventInfo.run(runIdx).onsets;
    durations = cfg.eventInfo.run(runIdx).durations;
    pmod = [];

    % create list with regressor-indices that will be deleted at the end of this function
    deleteRegIds = [];

    for gridEventTypeIdx = 1:length(cfg.eventInfo.run(runIdx).gridEventType)

        currentRegNr = find(strcmp(names, cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).eventName));

        % create logical indices for events that are used for the current GLM
        eventsForGridAnalysis_logIds = logical(cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).useEventForGLM(cfg.GLMnr).logicalIds);

        % add a new regressor at the end of the regressors-list, with the events that are used for the grid analysis
        if any(eventsForGridAnalysis_logIds)
            regNr_allGridEvents = length(names)+1;
            onsets{regNr_allGridEvents} = onsets{currentRegNr}(eventsForGridAnalysis_logIds);
            durations{regNr_allGridEvents} = durations{currentRegNr}(eventsForGridAnalysis_logIds);
            names(regNr_allGridEvents) = {['GridEvent-' names{currentRegNr}]};
        end

        % for the original regressor, keep just the events that are not used for grid analysis
        onsets{currentRegNr}(eventsForGridAnalysis_logIds) = [];
        durations{currentRegNr}(eventsForGridAnalysis_logIds) = [];
        names(currentRegNr) = {['UnusedGridEvent-' names{currentRegNr}]};    

        % delete the original regressor at the end of this function, if it is empty or not supposed to be used
        if all(eventsForGridAnalysis_logIds) || ~cfg.GLM.keepUnusedGridEvents
            deleteRegIds(end+1) = currentRegNr;
        end

        if any(eventsForGridAnalysis_logIds)

            % GLM1
            if cfg.GLMnr == 1

                % define the parametric modulation (SIN) for the grid event regressor
                pmod(regNr_allGridEvents).name{1} = '-pmodSIN';
                pmod(regNr_allGridEvents).param{1} = sind(cfg.GLM.xFoldSymmetry*cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).orientations(eventsForGridAnalysis_logIds));
                pmod(regNr_allGridEvents).poly{1} = 1;   

                % define pmod parametric modulation (COS) for the grid event regressor
                pmod(regNr_allGridEvents).name{2} = '-pmodCOS';
                pmod(regNr_allGridEvents).param{2} = cosd(cfg.GLM.xFoldSymmetry*cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).orientations(eventsForGridAnalysis_logIds));
                pmod(regNr_allGridEvents).poly{2} = 1;

            % GLM2
            elseif cfg.GLMnr == 2

                % transform grid event orientations from deg to rad
                orientations_rad = cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).orientations(eventsForGridAnalysis_logIds) * pi / 180;

                % load results (i.e. voxelwise orientation information) from GLM1
                GLM1_results = load([cfg.GLM.GLM1_resultsDir filesep 'GridCAT_GLM1.mat']);

                % test of estimated grid orientation is done for all voxels.
                % a ROI must be specified, however, from which the mean grid orientation is calculated and used to perform the test.
                for maskIdx = 1:length(cfg.GLM.GLM2_roiMask_calcMeanGridOri)
                    
                    % find the grid event with the appropriate name in GLM1 (since the gridEventTypeIdx could be different in GLM1 vs. GLM2)
                    gridEventTypeIdxInGLM1 = find(strcmp(cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).eventName, {GLM1_results.gridEventType(:).eventName})); 
                    
                    % calculate the mean grid orientation for the current ROI, using voxelwise orientations estimated after GLM1
                    if ~cfg.GLM.GLM2_averageMeanGridOriAcrossRuns
                        % mean grid ori is calculated for each run separately
                        meanGridOrientation_rad = calcMeanGridOri(cfg.GLM.xFoldSymmetry, cfg.GLM.GLM2_roiMask_calcMeanGridOri{maskIdx}, GLM1_results.gridEventType(gridEventTypeIdxInGLM1).run(runIdx), cfg.GLM.GLM2_useWeightingForVoxels);
                    else
                        % mean grid ori is calculated across all runs
                        meanGridOrientation_rad = calcMeanGridOri(cfg.GLM.xFoldSymmetry, cfg.GLM.GLM2_roiMask_calcMeanGridOri{maskIdx}, GLM1_results.gridEventType(gridEventTypeIdxInGLM1).allRunsAvg, cfg.GLM.GLM2_useWeightingForVoxels);
                    end
                    
                    % Did the mean grid orientation calculation lead to a valid result?
                    if isnan(meanGridOrientation_rad)
                        
                        warning(['A valid mean grid orientation cannot be caluclated for grid event ''' cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).eventName ...
                            ''' in run ' num2str(runIdx) '. Potential reasons: You have no valid voxels within your ROI mask for GLM2 mean grid ori calculation. ' ...
                            'Or you are attempting to estimate the mean grid orientation in GLM2 for runs, for which you do not have estimated voxel-wise grid orientations in GLM1. ' ...
                            'This only works, if you have multiple runs with the same grid event and you have selected to average the mean grid orientation across runs.']);
                    
                    else

                        % calculate a measure of alignment between a single event's orientation and the mean grid orientation
                        alignmentMeasure = cos(cfg.GLM.xFoldSymmetry*(orientations_rad - meanGridOrientation_rad));

                        if strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'pmod')

                            % define the parametric modulation with orientation for the grid event regressor
                            pmod(regNr_allGridEvents).name{maskIdx} = ['-PMOD-alignmentWithROI' num2str(maskIdx) 'meanOri'];
                            pmod(regNr_allGridEvents).param{maskIdx} = alignmentMeasure;
                            pmod(regNr_allGridEvents).poly{maskIdx} = 1;               

                        elseif strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'aligned_misaligned')

                            % create logical indices for aligned orientations
                            alignedEvents_logIds = alignmentMeasure >= 0;

                            % add a new regressor at the end of the regressors-list, with the ALIGNED events
                            regNr_alignedEvents = length(names)+1;
                            onsets{regNr_alignedEvents} = onsets{regNr_allGridEvents}(alignedEvents_logIds);
                            durations{regNr_alignedEvents} = durations{regNr_allGridEvents}(alignedEvents_logIds);
                            names(regNr_alignedEvents) = {[names{regNr_allGridEvents} '-alignedWithROI' num2str(maskIdx) 'meanOri']};                 

                            % add a new regressor at the end of the regressors-list, with the MISALIGNED events
                            regNr_misalignedEvents = length(names)+1;
                            onsets{regNr_misalignedEvents} = onsets{regNr_allGridEvents}(~alignedEvents_logIds);
                            durations{regNr_misalignedEvents} = durations{regNr_allGridEvents}(~alignedEvents_logIds);
                            names(regNr_misalignedEvents) = {[names{regNr_allGridEvents} '-misalignedWithROI' num2str(maskIdx) 'meanOri']};

                            % the regressor that contains all the grid events should be deleted, since now it is split up
                            % into two new regressors (per mask) with just aligned and misaligned events, respectively.
                            deleteRegIds(end+1) = regNr_allGridEvents;

                        elseif strcmp(cfg.GLM.GLM2_gridRegressorMethod, 'aligned_misaligned_multiple')

                            % at which orientation values (currentOri-meanOri) do we expect peaks in activation (both positive and negative peaks)?
                            expectedPeaks = 0 : 2*pi/(cfg.GLM.xFoldSymmetry*2) : 2*pi;
                            expectedPeaks(end) = []; % first peak (0 degrees) = last peak (360 degrees)

                            % determine, to which grid event regressor number (ids) each event belongs
                            ids = [];
                            for eventIdx = 1:length(orientations_rad)
                                [~, ids(eventIdx)] = min(absAngDiff(orientations_rad(eventIdx)-meanGridOrientation_rad, expectedPeaks, 2*pi));
                            end

                            % assign grid events to new regressors
                            for peakIdx = 1:length(expectedPeaks)
                                newRegNr = length(names)+1;
                                onsets{newRegNr} = onsets{regNr_allGridEvents}(ids==peakIdx);
                                durations{newRegNr} = durations{regNr_allGridEvents}(ids==peakIdx);
                                if mod(peakIdx,2); alignmentStr = 'aligned'; else; alignmentStr = 'misaligned'; end
                                names(newRegNr) = {[names{regNr_allGridEvents} '-' num2str(round(expectedPeaks(peakIdx)*180/pi)) 'degPeak-' alignmentStr 'WithROI' num2str(maskIdx) 'meanOri']};      
                            end

                            % the regressor that contains all the grid events should be deleted, since now it is split up
                            % into multiple new regressors (per mask) with aligned or unaligned orientations at different peak orientations
                            deleteRegIds(end+1) = regNr_allGridEvents;

                        end
                    end
                end
            end
        end    
    end

    % check whether there are regressors with not a single event (these must be deleted, because SPM will not accept it).
    emptyRegIds = find(cellfun(@isempty,onsets));
    for emptyRegIdx = emptyRegIds
        if ~any(emptyRegIdx==deleteRegIds)
            disp(['Run' num2str(runIdx) ': Regressor ''' names{emptyRegIdx} ''' is not included in the model because it does not contain any event.']);
        end
    end
    deleteRegIds = [deleteRegIds emptyRegIds];
    
    % delete unused regressors
    deleteRegIds = unique(deleteRegIds);
    names(deleteRegIds) = [];
    onsets(deleteRegIds) = [];
    durations(deleteRegIds) = [];
    if ~isempty(pmod)
        pmod(deleteRegIds) = [];
    end

    % set output directory
    outputDir = cfg.GLM.dataDir;
    if ~exist(outputDir, 'dir')
        mkdir(outputDir);
    end

    % Save multiple conditions info (including parametric modulations) in SPM format
    save([outputDir filesep 'multipleConditions_run' num2str(runIdx) '.mat'], 'names', 'onsets', 'durations', 'pmod');
    
end




