function gridMetric_export(roiMasks, GLM1dir, GLM2dir, saveToPath)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function exports all grid code metrics that can be calculated for a dataset

% ------------------
% settings

sep = ';';
totalWaitbarSteps = 4;

% ------------------

outputLine = {};

% specify destination of output data, if it wasn't specified at function call
if nargin<4
    saveToPath = selectionDlg_GUI({'putFile'}, 'Save to...', pwd, {'*'}, 0, [1 1], 'on', 0, 'GridCAT - main figure', {'.txt'}, 'on', 'GridCAT_GridMetrics_output.txt');
end
if isempty(saveToPath)
    return
end

% Create waitbar
waitbarStep = 0;
if usejava('jvm')
    hWaitbar = waitbar(waitbarStep/totalWaitbarSteps, 'Grid metrics are exported. Please wait...');
end


%% Magnitude of grid code response within ROI
      
% Check whether a GridCAT_GLM2.mat file can be found, as an indicator of whether
% GLM2 has been computed and a valid GLM2 data directory has been specified
if exist([GLM2dir filesep 'GridCAT_GLM2.mat'], 'file')

    % calculate grid metrics
    output = gridMetric_gridCodeResponseMagnitude(0, roiMasks, GLM2dir);

    % which xFoldSymmetry are the calculations testing for?
    GLM2output = load([GLM2dir filesep 'GridCAT_GLM2.mat']);
    xFoldSymmetry = GLM2output.cfg.GLM.xFoldSymmetry;

    % add headline to output string
    outputLine{end+1} = ['GRID METRIC' sep 'X-FOLD SYMMETRY' sep 'ROI' sep 'CONTRAST NUMBER' sep 'CONTRAST NAME' sep 'MEAN CON-VALUE WITHIN ROI' sep 'VOXELS WITHIN ROI' sep 'NaN VOXELS WITHIN ROI'];

    % add grid metrics to output string
    for maskIdx = 1:length(output.mask)
        for conIdx = 1:length(output.mask(maskIdx).contrast)
            outputLine{end+1} = [ ...
                'Magnitude of grid code response within ROI' sep ...
                num2str(xFoldSymmetry) sep ... xFoldSymmetry
                output.mask(maskIdx).maskFile sep ... % mask file
                num2str(conIdx) sep ... % contrast number
                output.mask(maskIdx).contrast(conIdx).contrastName sep ... % contrast name
                sprintf('%.3f', output.mask(maskIdx).contrast(conIdx).meanConValueWithinMask) sep ... % mean con-value within mask
                num2str(output.mask(maskIdx).contrast(conIdx).nVoxelsWithinMask) sep ... % voxels within mask
                num2str(output.mask(maskIdx).contrast(conIdx).nNanVoxelsWithinMask) sep ... % NaN voxels within mask
                ];
        end
    end
end

% Update waitbar
waitbarStep = waitbarStep+1;
if usejava('jvm')
    waitbar(waitbarStep/totalWaitbarSteps);
end


%% Between-voxel grid orientation coherence within ROI

% Check whether a GridCAT_GLM1.mat file can be found, as an indicator of whether
% GLM1 has been computed and a valid GLM1 data directory has been specified
if exist([GLM1dir filesep 'GridCAT_GLM1.mat'], 'file')

    % get a list of all available images with voxelwise grid orientations
    voxelwiseOriImages = dir([GLM1dir filesep 'voxelwiseGridOri*.nii']);
    voxelwiseOriImages = {voxelwiseOriImages(:).name};
    voxelwiseOriImages = strcat(GLM1dir, filesep, voxelwiseOriImages);

    % which xFoldSymmetry are the calculations testing for?
    GLM1output = load([GLM1dir filesep 'GridCAT_GLM1.mat']);
    xFoldSymmetry = GLM1output.cfg.GLM.xFoldSymmetry;

    % calculate grid metrics
    output = gridMetric_oriCoherence_betweenVox(0, roiMasks, voxelwiseOriImages, xFoldSymmetry);

    % add headline to output string
    outputLine{end+1} = '';
    outputLine{end+1} = ['GRID METRIC' sep 'X-FOLD SYMMETRY'  sep 'ROI' sep 'VOXEL-WISE GRID ORI IMAGE' sep 'RAYLEIGH z' sep 'RAYLEIGH p' sep 'VOXELS WITHIN ROI' sep 'NaN VOXELS WITHIN ROI'];

    % add grid metrics to output string
    for maskIdx = 1:length(output.mask)
        for imgIdx = 1:length(output.mask(maskIdx).voxelwiseOriImage)
            outputLine{end+1} = [ ...
                'Between-voxel grid orientation coherence within ROI' sep ...
                num2str(xFoldSymmetry) sep ... xFoldSymmetry
                output.mask(maskIdx).maskFile sep ... % mask file
                output.mask(maskIdx).voxelwiseOriImage(imgIdx).imageName sep ... % voxelwise grid ori image
                sprintf('%.3f', output.mask(maskIdx).voxelwiseOriImage(imgIdx).rayleigh_z) sep ... % rayleigh z
                sprintf('%.3f', output.mask(maskIdx).voxelwiseOriImage(imgIdx).rayleigh_p) sep ... % rayleigh p
                num2str(output.mask(maskIdx).voxelwiseOriImage(imgIdx).nVoxelsWithinMask) sep ... % voxels within mask
                num2str(output.mask(maskIdx).voxelwiseOriImage(imgIdx).nNanVoxelsWithinMask) sep ... % NaN voxels within mask
                ];
        end
    end
end

% Update waitbar
waitbarStep = waitbarStep+1;
if usejava('jvm')
    waitbar(waitbarStep/totalWaitbarSteps);
end


%% Within-voxel grid orientation coherence within ROI

% Check whether a GridCAT_GLM1.mat file can be found, as an indicator of whether
% GLM1 has been computed and a valid GLM1 data directory has been specified
if exist([GLM1dir filesep 'GridCAT_GLM1.mat'], 'file')

    % get a list of all available images with voxelwise grid orientations
    voxelwiseOriImages = dir([GLM1dir filesep 'voxelwiseGridOri*.nii']);
    voxelwiseOriImages = {voxelwiseOriImages(:).name};
    voxelwiseOriImages = strcat(GLM1dir, filesep, voxelwiseOriImages);

    % which xFoldSymmetry are the calculations testing for?
    GLM1output = load([GLM1dir filesep 'GridCAT_GLM1.mat']);
    xFoldSymmetry = GLM1output.cfg.GLM.xFoldSymmetry;

    % add headline to output string
    outputLine{end+1} = '';
    outputLine{end+1} = ['GRID METRIC' sep 'X-FOLD SYMMETRY'  sep 'ROI' sep 'VOXEL-WISE GRID ORI IMAGE 1' sep 'VOXEL-WISE GRID ORI IMAGE 2' sep '% STABLE VOXELS WITHIN ROI' sep 'IMAGE 1 VOXELS WITHIN ROI' sep 'IMAGE 2 VOXELS WITHIN ROI' sep 'IMAGE 1 NaN VOXELS WITHIN ROI' sep 'IMAGE 2 NaN VOXELS WITHIN ROI'];

    % calculate grid metrics for all possible pairs of voxelwise images
    for img1Idx = 1:length(voxelwiseOriImages)
        for img2Idx = (img1Idx+1):length(voxelwiseOriImages)

            % calculate grid metrics for the current pair of voxelwise images
            imgPair = voxelwiseOriImages([img1Idx img2Idx]);
            output = gridMetric_oriCoherence_withinVox(0, roiMasks, imgPair, xFoldSymmetry, 360/xFoldSymmetry/4);

            % add grid metrics to output string
            for maskIdx = 1:length(output.mask)
                outputLine{end+1} = [ ...
                    'Within-voxel grid orientation coherence within ROI' sep ...
                    num2str(xFoldSymmetry) sep ... xFoldSymmetry
                    output.mask(maskIdx).maskFile sep ... % mask file
                    output.mask(maskIdx).voxelwiseOriImage(1).imageName sep ... % voxelwise grid ori image1
                    output.mask(maskIdx).voxelwiseOriImage(2).imageName sep ... % voxelwise grid ori image2
                    sprintf('%.2f', output.mask(maskIdx).percentageStableVoxels*100) sep ... % percentage of stable voxels within mask
                    num2str(output.mask(maskIdx).voxelwiseOriImage(1).nVoxelsWithinMask) sep ... % image1 voxels within mask
                    num2str(output.mask(maskIdx).voxelwiseOriImage(2).nVoxelsWithinMask) sep ... % image2 voxels within mask
                    num2str(output.mask(maskIdx).voxelwiseOriImage(1).nNanVoxelsWithinMask) sep ... % image1 NaN voxels within mask
                    num2str(output.mask(maskIdx).voxelwiseOriImage(2).nNanVoxelsWithinMask) sep ... % image2 NaN voxels within mask
                    ];
            end        
        end
    end
end

% Update waitbar
waitbarStep = waitbarStep+1;
if usejava('jvm')
    waitbar(waitbarStep/totalWaitbarSteps);
end


%% Mean grid orientation within ROI

% Check whether a GridCAT_GLM1.mat file can be found, as an indicator of whether
% GLM1 has been computed and a valid GLM1 data directory has been specified
if exist([GLM1dir filesep 'GridCAT_GLM1.mat'], 'file')

    % Load GLM1 output
    GLM1output = load([GLM1dir filesep 'GridCAT_GLM1.mat']);

    % which xFoldSymmetry are the calculations testing for?
    xFoldSymmetry = GLM1output.cfg.GLM.xFoldSymmetry;

    % Calculate mean grid orientation within ROI for all grid events that are stored in GridCAT_GLM1.mat
    output = gridMetric_meanGridOriWithinROI(0, roiMasks, GLM1output);

    % add headline to output string
    outputLine{end+1} = '';
    outputLine{end+1} = ['GRID METRIC' sep 'X-FOLD SYMMETRY' sep 'ROI' sep 'GRID EVENT' sep 'RUN' sep 'MEAN GRID ORI IN DEGREES (VOXELS WEIGHTED)' sep 'MEAN GRID ORI IN DEGREES (VOXELS NOT WEIGHTED)'];

    % add grid metrics to output string
    for maskIdx = 1:length(output.mask)
        for gridEventTypeIdx = 1:length(output.mask(maskIdx).gridEventType)
            for runIdx = 1:length(output.mask(maskIdx).gridEventType(gridEventTypeIdx).run)

                % get metrics for individual run
                outputLine{end+1} = [ ...
                    'Mean grid orientation within ROI' sep ...
                    num2str(xFoldSymmetry) sep ... xFoldSymmetry
                    output.mask(maskIdx).maskFile sep ... % mask file
                    output.mask(maskIdx).gridEventType(gridEventTypeIdx).gridEventName sep ... % grid event name
                    num2str(runIdx) sep ... % run number
                    sprintf('%.3f', output.mask(maskIdx).gridEventType(gridEventTypeIdx).run(runIdx).meanGridOri_weighted_rad*180/pi) sep ... % mean grid ori in degrees (voxels weighted)
                    sprintf('%.3f', output.mask(maskIdx).gridEventType(gridEventTypeIdx).run(runIdx).meanGridOri_notWeighted_rad*180/pi) sep ... % mean grid ori in degrees (voxels not weighted)
                    ];
            end

            % get metrics averaged across runs
            outputLine{end+1} = [ ...
                'Mean grid orientation within ROI' sep ...
                num2str(xFoldSymmetry) sep ... xFoldSymmetry
                output.mask(maskIdx).maskFile sep ... % mask file
                output.mask(maskIdx).gridEventType(gridEventTypeIdx).gridEventName sep ... % grid event name
                'averaged across runs' sep ... % run number
                sprintf('%.3f', output.mask(maskIdx).gridEventType(gridEventTypeIdx).allRunsAvg.meanGridOri_weighted_rad*180/pi) sep ... % mean grid ori in degrees (voxels weighted)
                sprintf('%.3f', output.mask(maskIdx).gridEventType(gridEventTypeIdx).allRunsAvg.meanGridOri_notWeighted_rad*180/pi) sep ... % mean grid ori in degrees (voxels not weighted)
                ];
        end
    end
end

% Update waitbar
waitbarStep = waitbarStep+1;
if usejava('jvm')
    waitbar(waitbarStep/totalWaitbarSteps);
end


%% SAVE METRICS TO FILE
                  
[fpath, fname, fext] = fileparts(saveToPath{1});
saveCellArrayToTextfile(outputLine, fpath, [fname fext], 0);

% close waitbar
close(hWaitbar);



