function outputData = gridMetric_oriCoherence_betweenVox(dispOutput_flag, mask_images, voxelwiseOri_images, xFoldSymmetry, plotMeanOri_unweightedWeighted, handle_drawInAxes)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function calculates and visualizes the between-voxel coherence of
% grid orientations within one or multiple ROIs.

for imgIdx = 1:length(voxelwiseOri_images)
    
    % load image. voxelwise orientations are in range [0 60] in degrees.
    [~, voxelwiseOri_vol_deg] = loadImage_SPM(voxelwiseOri_images{imgIdx});

    % transform grid ori from range [0 360/xFoldSymmetry] to [0 360] degrees.
    voxelwiseOri_vol_deg = voxelwiseOri_vol_deg * xFoldSymmetry;     
    
    for maskIdx = 1:length(mask_images)

        % load mask
        [~, mask_vol] = loadImage_SPM(mask_images{maskIdx});

        % get values that are within the ROI mask and not NaN
        valuesWithinMask_deg = voxelwiseOri_vol_deg(mask_vol==1);
        nVoxelsWithinMask = length(valuesWithinMask_deg);
        nNanVoxelsWithinMask = sum(isnan(valuesWithinMask_deg));
        valuesWithinMask_deg(isnan(valuesWithinMask_deg)) = [];
        
        % calculate Rayleigh test for non-uniformity of circular data
        [rayleigh_p, rayleigh_z] = circ_rtest(valuesWithinMask_deg*pi/180);
        
        % get filenames of mask and voxelwise ori image
        [~, mask_fname, mask_fext] = fileparts(mask_images{maskIdx});
        mask_fname(mask_fname=='_') = '-';    
        [~, img_fname, img_fext] = fileparts(voxelwiseOri_images{imgIdx});
        img_fname(img_fname=='_') = '-'; 
        
        if dispOutput_flag
            
            if exist('handle_drawInAxes') && ishandle(handle_drawInAxes)
                axes(handle_drawInAxes);
            else

                % create new figure
                hfig = figure();

                % adapt position of the new figure
                adaptFigurePosition(hfig);
            end            

            % plot the values
            hrose = rose(valuesWithinMask_deg*pi/180); % rose function expects [rad]
            set(hrose, 'LineWidth', 2);

            % change labels of the polar plot to a range [0 360/xFoldSymmetry]
            hHiddenText = findall(gca,'type','text');
            plotAngles = 0:30:330;
            for ang = plotAngles
               hObj = findall(hHiddenText,'String',num2str(ang));
               set(hObj, 'String', num2str(ang/xFoldSymmetry));
            end
            
%             % create some extra space at the top of the plot
%             xlimits = xlim;
%             ylimits = ylim;
%             ylim([ylimits(1) ylimits(2)*1.3]);

%             % plot text on graph
%             visStr = sprintf('ROI mask: %s %s\nVoxel-wise ori image: %s %s', mask_fname, mask_fext, img_fname, img_fext);
%             text(0, ylimits(2)*1.3, visStr , 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center');
%             text(xlimits(1), ylimits(1), sprintf('z=%.3f', rayleigh_z), 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center'); 
%             text(xlimits(2), ylimits(1), sprintf('p=%.3f', rayleigh_p), 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center'); 
%             text(xlimits(1), ylimits(2), sprintf('%i-fold symmetry', xFoldSymmetry), 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold'); 

            % Plot arrows for mean grid orientation with and/or without amplitude based weighting of voxels
            if exist('plotMeanOri_unweightedWeighted', 'var') && sum(plotMeanOri_unweightedWeighted)
                
                % find and load voxel-wise amplitude image corresponding to current voxe-wise grid ori image
                [fpat, fnam, fext] = fileparts(voxelwiseOri_images{imgIdx});
                fnamParts = strsplit(fnam, '_');
                amplitudeImage = [fpat filesep 'voxelwiseAmplitude_' fnamParts{2} '_' fnamParts{3} fext];
                [~, voxelwiseAmp_vol] = loadImage_SPM(amplitudeImage);
 
                % create structure with ori and amplitude, as needed for calcMeanGridOri fcn
                GLM1_results.amplitudePerVoxel = voxelwiseAmp_vol;
                GLM1_results.oriPerVoxel_rad = (voxelwiseOri_vol_deg*pi/180) / xFoldSymmetry; % fcn expects [rad]
           
                hold on;
                arrowLen=xlim;         
            
                % Plot mean grid orientations without weighting
                if plotMeanOri_unweightedWeighted(1)

                    arrowOri_rad = calcMeanGridOri(xFoldSymmetry, mask_images{maskIdx}, GLM1_results, 0);
                    hArrow = compass(arrowLen(2)*cos(arrowOri_rad*xFoldSymmetry), arrowLen(2)*sin(arrowOri_rad*xFoldSymmetry));
                    set(hArrow, 'LineWidth', 4, 'Color', [1, 0, 0], 'LineStyle','-');
                    
                    % save mean grid ori on output data
                    outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).meanGridOri_unweighted = arrowOri_rad;

                end

                % Plot mean grid orientations with weighting
                if plotMeanOri_unweightedWeighted(2)

                    arrowOri_rad = calcMeanGridOri(xFoldSymmetry, mask_images{maskIdx}, GLM1_results, 1);
                    hArrow = compass(arrowLen(2)*cos(arrowOri_rad*xFoldSymmetry), arrowLen(2)*sin(arrowOri_rad*xFoldSymmetry));
                    set(hArrow, 'LineWidth', 4, 'Color', [0, 1, 0], 'LineStyle','-');
                    
                    % save mean grid ori on output data
                    outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).meanGridOri_weighted = arrowOri_rad;                
                    
                end

                hold off
                
            end
        end
            
        % write data to output structure
        outputData.mask(maskIdx).maskFile = [mask_fname, mask_fext];
        outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).imageName = [img_fname, img_fext];
        outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).rayleigh_z = rayleigh_z;
        outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).rayleigh_p = rayleigh_p;
        outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).nVoxelsWithinMask = nVoxelsWithinMask;
        outputData.mask(maskIdx).voxelwiseOriImage(imgIdx).nNanVoxelsWithinMask = nNanVoxelsWithinMask;

    end
end

drawnow
