function outputData = gridMetric_oriCoherence_withinVox(dispOutput_flag, mask_images, voxelwiseOri_images, xFoldSymmetry, stabilityThreshold_deg, handle_drawInAxes, handle_outputText, lineWidth_stableVox, lineWidth_unstabVox, lineColor_stableVox, lineColor_unstabVox, lineStyle_stableVox, lineStyle_unstabVox)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function calculates and visualizes the within-voxel coherence of
% grid orientations across two voxel-wise orientation images, within one or multiple ROIs.

% default settings:
if nargin < 6
    lineWidth_stableVox = 3;
    lineWidth_unstabVox = 3;
    lineColor_stableVox = [0.2 0.5 0.3];
    lineColor_unstabVox = [1 0 0];
    lineStyle_stableVox = '-';
    lineStyle_unstabVox = ':';
end


for maskIdx = 1:length(mask_images)
    
    % load mask
    [~, mask_vol] = loadImage_SPM(mask_images{maskIdx});


    %% calculations

    img = [];
    for imgIdx = 1:length(voxelwiseOri_images)

        % load image. voxelwise orientations are in range [0 60] in degrees.
        [~, voxelwiseOri_vol] = loadImage_SPM(voxelwiseOri_images{imgIdx});

        % transform grid ori from range [0 360/xFoldSymmetry] to [0 360] degrees.
        voxelwiseOri_vol = voxelwiseOri_vol * xFoldSymmetry;

        % get values that are within the ROI mask and not NaN
        img(imgIdx).anglesWithinMask = voxelwiseOri_vol(mask_vol==1);
        img(imgIdx).nVoxelsWithinMask = length(img(imgIdx).anglesWithinMask);
        img(imgIdx).nNanVoxelsWithinMask = sum(isnan(img(imgIdx).anglesWithinMask));
        img(imgIdx).anglesWithinMask(isnan(img(imgIdx).anglesWithinMask)) = [];

        % get from origin to endpoint, via orientation and length:
        % endpoint_x = origin_x + length * cos(angle_rad)
        % endpoint_y = origin_y + length * sin(angle_rad)
        img(imgIdx).markerPos_xy(:,1) = imgIdx*cosd(img(imgIdx).anglesWithinMask);
        img(imgIdx).markerPos_xy(:,2) = imgIdx*sind(img(imgIdx).anglesWithinMask);

    end
    
    % which voxels are 'stable' between conditions, i.e., have the same orientation +/- stabilityThreshold
    voxIds_stableOri = absAngDiff(img(1).anglesWithinMask, img(2).anglesWithinMask, 360) <= (stabilityThreshold_deg * xFoldSymmetry);

    % calculate percentage of 'stable' voxels
    percentageOfStableVoxels = sum(voxIds_stableOri)/length(voxIds_stableOri);
    
    % Write data to output structure
    [~, mask_fname, mask_fext] = fileparts(mask_images{maskIdx});
    [~, imgA_fname, imgA_fext] = fileparts(voxelwiseOri_images{1}); % inner image in plot
    [~, imgB_fname, imgB_fext] = fileparts(voxelwiseOri_images{2}); % outer image in plot
    outputData.mask(maskIdx).maskFile = [mask_fname, mask_fext];
    outputData.mask(maskIdx).voxelwiseOriImage(1).imageName = [imgA_fname, imgA_fext];
    outputData.mask(maskIdx).voxelwiseOriImage(2).imageName = [imgB_fname, imgB_fext];
    outputData.mask(maskIdx).voxelwiseOriImage(1).nVoxelsWithinMask = img(1).nVoxelsWithinMask;
    outputData.mask(maskIdx).voxelwiseOriImage(2).nVoxelsWithinMask = img(2).nVoxelsWithinMask;
    outputData.mask(maskIdx).voxelwiseOriImage(1).nNanVoxelsWithinMask = img(1).nNanVoxelsWithinMask;
    outputData.mask(maskIdx).voxelwiseOriImage(2).nNanVoxelsWithinMask = img(2).nNanVoxelsWithinMask;
    outputData.mask(maskIdx).percentageStableVoxels = percentageOfStableVoxels;
    outputData.mask(maskIdx).stabilityThreshold_deg = stabilityThreshold_deg;


    %% plotting

    if dispOutput_flag
        
        if exist('handle_drawInAxes') && ishandle(handle_drawInAxes)
            axes(handle_drawInAxes);
        else
            
            % create new figure
            hfig = figure();
            
            % adapt position of the new figure
            adaptFigurePosition(hfig);
        end

        axis off

        for imgIdx = length(voxelwiseOri_images):-1:1 % descending order, to start with the biggest circle and avoid resizing of the figure

            % plot the circles    
            pos = [-imgIdx -imgIdx imgIdx*2 imgIdx*2];
            rectangle('Position',pos,'Curvature',[1 1])
            axis equal
            hold on

        end

        % plot UNSTALBE voxels' orientation change
        line([img(imgIdx).markerPos_xy(~voxIds_stableOri,1), img(imgIdx+1).markerPos_xy(~voxIds_stableOri,1)]', [img(imgIdx).markerPos_xy(~voxIds_stableOri,2), img(imgIdx+1).markerPos_xy(~voxIds_stableOri,2)]', 'Color', lineColor_unstabVox, 'LineWidth', lineWidth_unstabVox, 'LineStyle', lineStyle_unstabVox);
        
        % plot STABLE voxels' orientations change
        line([img(imgIdx).markerPos_xy(voxIds_stableOri,1), img(imgIdx+1).markerPos_xy(voxIds_stableOri,1)]', [img(imgIdx).markerPos_xy(voxIds_stableOri,2), img(imgIdx+1).markerPos_xy(voxIds_stableOri,2)]',     'Color', lineColor_stableVox, 'LineWidth', lineWidth_stableVox, 'LineStyle', lineStyle_stableVox);
        
        % plot proportion of stable voxels
        if exist('handle_outputText') && ishandle(handle_outputText)
            set(handle_outputText, 'String', [num2str(round(percentageOfStableVoxels*10000)/100) '%']);
        end

        for imgIdx = 1:length(voxelwiseOri_images)

            % plot markers for each UNSTABLE voxel's orientation
            plot(img(imgIdx).markerPos_xy(~voxIds_stableOri,1),img(imgIdx).markerPos_xy(~voxIds_stableOri,2),'o', 'MarkerSize', 10, 'MarkerEdgeColor',[0 0 0], 'MarkerFaceColor',lineColor_unstabVox);       

            % plot markers for each STABLE voxel's orientation
            plot(img(imgIdx).markerPos_xy(voxIds_stableOri,1),img(imgIdx).markerPos_xy(voxIds_stableOri,2),'o', 'MarkerSize', 10, 'MarkerEdgeColor',[0 0 0], 'MarkerFaceColor',lineColor_stableVox);       
        
        end
        
    end
    
end