function [eventInfo] = readEventTable(eventTable_file)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% This function reads an event-table and converts the data into the correct
% format for grid code analyis using the GridCAT.
% This function accepts the following delimiters within event-tables:
%       - semicolon ';'
%       - colon ','
%       - tabstop char(9)

eventTableError = false; % default init

try
    
    % Read data from event-table file
    eventTableRawData = [fileread(eventTable_file) char(13)];

    % The function expects the data to be delimited by semicolon ';'
    % However, the event-table file might be specified using a different delimiter.
    % For this reason, we convert different potential delimiters (e.g. ',') to semicolons.
    eventTableRawData(eventTableRawData==char(9))=';'; % char(9) = tab
    eventTableRawData(eventTableRawData==',')=';';

    % now the data should definitely be delimited by semicolons and can be formatted
    eventTableRawData = textscan(eventTableRawData, '%s %f %f %f %f %f',  'delimiter', ';');

    % sort lines/events according to their onset times
    [eventOnsets, eventOrderIds] = sort(eventTableRawData{2});
    eventNames = eventTableRawData{1}(eventOrderIds);
    eventDurations = eventTableRawData{3}(eventOrderIds);
    eventOrientations = eventTableRawData{4}(eventOrderIds);
    useEventForGridAnalysis_GLM1 = eventTableRawData{5}(eventOrderIds);
    useEventForGridAnalysis_GLM2 = eventTableRawData{6}(eventOrderIds);

    % transpose input if neccessary
    if size(eventOnsets,1)>1 && size(eventOnsets,2)==1
        eventOnsets = eventOnsets';
    end
    if size(eventNames,1)>1 && size(eventNames,2)==1
        eventNames = eventNames';
    end
    if size(eventDurations,1)>1 && size(eventDurations,2)==1
        eventDurations = eventDurations';
    end
    if size(eventOrientations,1)>1 && size(eventOrientations,2)==1
        eventOrientations = eventOrientations';
    end
    if size(useEventForGridAnalysis_GLM1,1)>1 && size(useEventForGridAnalysis_GLM1,2)==1
        useEventForGridAnalysis_GLM1 = useEventForGridAnalysis_GLM1';
    end
    if size(useEventForGridAnalysis_GLM2,1)>1 && size(useEventForGridAnalysis_GLM2,2)==1
        useEventForGridAnalysis_GLM2 = useEventForGridAnalysis_GLM2';
    end
    
catch
    
    eventTableError = true;
    
end

% Perform some sanity checks on the data that has been read
if eventTableError || any(isnan(eventOnsets)) || any(isnan(eventDurations))
    error(['The following event-table seems to have a wrong format and could not be read: ' eventTable_file]);
end

% create list of unique event names
eventInfo.names = unique(eventNames, 'stable'); % the 'stable' option does not work for older matlab releases. can be used instead: eventInfo.names = unique(eventNames);

% organize onsets and durations per event-name in SPM format
for eventTypeIdx = 1:length(eventInfo.names)    
    currentEventName_eventIds = strcmp(eventNames, eventInfo.names{eventTypeIdx});    
    eventInfo.onsets{eventTypeIdx} = eventOnsets(currentEventName_eventIds);
    eventInfo.durations{eventTypeIdx} = eventDurations(currentEventName_eventIds);
end

% Find grid events (which have a specified orientation information)
gridEventIds = ~isnan(eventOrientations);

% Get names of grid event regressors
gridEventNames = unique(eventNames(gridEventIds));

% Get infos about grid events
for eventTypeIdx = 1:length(gridEventNames)
    eventInfo.gridEventType(eventTypeIdx).eventName = gridEventNames{eventTypeIdx};
    currentEventName_eventIds = strcmpi(eventNames, eventInfo.gridEventType(eventTypeIdx).eventName);
    eventInfo.gridEventType(eventTypeIdx).orientations = eventOrientations(currentEventName_eventIds);
    eventInfo.gridEventType(eventTypeIdx).nEvents = length(eventInfo.gridEventType(eventTypeIdx).orientations);
    
    % Is it specified in the event-table, which event should be used for GLM1 (estimate grid ori) and/or GLM2 (test grid ori)?
    if ~all(isnan(useEventForGridAnalysis_GLM1)) || ~all(isnan(useEventForGridAnalysis_GLM2))

        % If it is specified, check whether the information is complete. If yes, save it.
        if isequal( ~isnan(useEventForGridAnalysis_GLM1), ~isnan(useEventForGridAnalysis_GLM2), gridEventIds )
            eventInfo.gridEventType(eventTypeIdx).useEventForGLM(1).logicalIds = logical(useEventForGridAnalysis_GLM1(currentEventName_eventIds));
            eventInfo.gridEventType(eventTypeIdx).useEventForGLM(2).logicalIds = logical(useEventForGridAnalysis_GLM2(currentEventName_eventIds));
        else
            error('For some but not all ''grid-events'' (the ones that have a defined orientation information) it is specified in the event-table, whether they are used in GLM1 and/or GLM2. Please either specify it for all or for none.');
        end
    else
        % If it is not specified, save an empty array.
        eventInfo.gridEventType(eventTypeIdx).useEventForGLM(1).logicalIds = [];
        eventInfo.gridEventType(eventTypeIdx).useEventForGLM(2).logicalIds = [];
    end
    
end


