function cfg = specifyEventUsageForGLM(cfg)
%
%     Copyright 2017 Matthias Stangl, Jonathan Shine, Thomas Wolbers
%     matthias.stangl@dzne.de, jonathan.shine@dzne.de, thomas.wolbers@dzne.de
% 
%     This file is part of the GridCAT.
% 
%     The GridCAT is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     The GridCAT is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with the GridCAT.  If not, see <http://www.gnu.org/licenses/>.
%
%
% Based on the variable eventUsageSpecifier, a subset of grid events is
% specified for usage in a GLM.
%
% eventUsageSpecifier:
%	2 ... use the first half of grid events per run
%	3 ... use the second half of grid events per run
%	4 ... use odd grid events within each run
%	5 ... use even grid events within each run
%	6 ... use all grid events of odd runs
%	7 ... use all grid events of even runs
%	8 ... use all grid events of all runs
%	9 ... use specification from event-table

% read data from cfg
eventUsageSpecifier = cfg.GLM.eventUsageSpecifier;

% specify usage of grid events for every run separately
for runIdx = 1:length(cfg.rawData.run)
   
    % specify usage of grid events for every grid event type separately
    for gridEventTypeIdx = 1:length(cfg.eventInfo.run(runIdx).gridEventType)
        
        % number of events for this run and event type
        nEvents = cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).nEvents;
        
        % delete useEvents-vector content if it already exists
        useEvents_logicalIds = [];
        
        % specify usage of grid events depending on event usage specifier (input parameter)
        switch eventUsageSpecifier
            
            case 2
        
                % use the first half of all events in a run
                useEvents_logicalIds(1:nEvents) = 0;
                useEvents_logicalIds(1:round(end/2)) = 1;

            case 3

                % use the second half of all events in a run
                useEvents_logicalIds(1:nEvents) = 1;
                useEvents_logicalIds(1:round(end/2)) = 0;

            case 4
            
                % use odd events of all runs
                useEvents_logicalIds(1:2:nEvents) = 1;
                useEvents_logicalIds(2:2:nEvents) = 0;
            
            case 5
            
                % use even events of all runs
                useEvents_logicalIds(1:2:nEvents) = 0;
                useEvents_logicalIds(2:2:nEvents) = 1;
            
            case 6
                
                % use all events of odd runs
                if mod(runIdx,2)
                    useEvents_logicalIds(1:nEvents) = 1;
                else
                    useEvents_logicalIds(1:nEvents) = 0;
                end
            
            case 7
            
                % use all events of even runs
                if mod(runIdx,2)
                    useEvents_logicalIds(1:nEvents) = 0;
                else
                    useEvents_logicalIds(1:nEvents) = 1;
                end           
            
            case 8
                
                % use all events of all runs
                useEvents_logicalIds(1:nEvents) = 1; 
                
            case 9
                
                % use specification from event-table
                % the variable on cfg is already set. so here, we can just perform checks whether they are ok (not empty etc.)
                useEvents_logicalIds = cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).useEventForGLM(cfg.GLMnr).logicalIds;
                if isempty(useEvents_logicalIds) || length(useEvents_logicalIds)~=nEvents
                    error('You have chosen to read from event-table which grid events to use as such in GLM1 and/or GLM2. However, the information in the event-table seems to be incorrect. Please check your event-table definitions.');
                end
        end
        
        % save event usage on cfg
        cfg.eventInfo.run(runIdx).gridEventType(gridEventTypeIdx).useEventForGLM(cfg.GLMnr).logicalIds = useEvents_logicalIds;
        
    end 
end
